@@ -39,6 +39,7 @@ else:
     except (ImportError, ValueError, GLib.Error):
         _nm_imported = False
 _nm_client = None
+_nm_client_timeout = None
 
 from firewall import errors
 from firewall.errors import FirewallError
@@ -61,9 +62,28 @@ def nm_get_client():
     """Returns the NM client object or None if the import of NM failed
     @return NM.Client instance if import was successful, None otherwise
     """
+
+    def _release():
+        """
+        Release the client to avoid excess memory usage when libnm pushes
+        irrelevant (to firewalld) updates.
+        """
+        global _nm_client
+        global _nm_client_timeout
+        _nm_client = None
+        _nm_client_timeout = None
+
     global _nm_client
+    global _nm_client_timeout
+
     if not _nm_client:
         _nm_client = NM.Client.new(None)
+    else:
+        # refresh timer
+        GLib.source_remove(_nm_client_timeout)
+
+    _nm_client_timeout = GLib.timeout_add_seconds(5, _release)
+
     return _nm_client
 
 def nm_get_zone_of_connection(connection):
@@ -75,21 +95,21 @@ def nm_get_zone_of_connection(connection):
 
     con = nm_get_client().get_connection_by_uuid(connection)
     if con is None:
-        return False
+        return None
 
     setting_con = con.get_setting_connection()
     if setting_con is None:
-        return False
+        return None
 
     try:
         if con.get_flags() & (NM.SettingsConnectionFlags.NM_GENERATED
                               | NM.SettingsConnectionFlags.NM_VOLATILE):
-            return False
+            return ""
     except AttributeError:
         # Prior to NetworkManager 1.12, we can only guess
         # that a connection was generated/volatile.
         if con.get_unsaved():
-            return False
+            return ""
 
     zone = setting_con.get_zone()
     if zone is None:
@@ -141,7 +161,9 @@ def nm_get_connections(connections, connections_name):
 
         connections_name[uuid] = name
         for dev in devices:
-            connections[dev.get_iface()] = uuid
+            ip_iface = dev.get_ip_iface()
+            if ip_iface:
+                connections[ip_iface] = uuid
 
 def nm_get_interfaces():
     """Get active interfaces from NM
@@ -169,7 +191,9 @@ def nm_get_interfaces():
                 continue
 
         for dev in active_con.get_devices():
-            active_interfaces.append(dev.get_iface())
+            ip_iface = dev.get_ip_iface()
+            if ip_iface:
+                active_interfaces.append(ip_iface)
 
     return active_interfaces
 
@@ -182,6 +206,22 @@ def nm_get_interfaces_in_zone(zone):
 
     return interfaces
 
+def nm_get_device_by_ip_iface(interface):
+    """Get device from NM which has the given IP interface
+    @param interface name
+    @returns NM.Device instance or None
+    """
+    check_nm_imported()
+
+    for device in nm_get_client().get_devices():
+        ip_iface = device.get_ip_iface()
+        if ip_iface is None:
+            continue
+        if ip_iface == interface:
+            return device
+
+    return None
+
 def nm_get_connection_of_interface(interface):
     """Get connection from NM that is using the interface
     @param interface name
@@ -189,7 +229,7 @@ def nm_get_connection_of_interface(interface):
     """
     check_nm_imported()
 
-    device = nm_get_client().get_device_by_iface(interface)
+    device = nm_get_device_by_ip_iface(interface)
     if device is None:
         return None
 
